from django.urls import include, path

urlpatterns = [
    path("", include("shopify_auth.urls")),
    path("session_tokens/", include("shopify_auth.session_tokens.urls", namespace="session_tokens")),
]



from django.urls import path

from . import views

app_name = "session_tokens"

urlpatterns = [
    path("finalize", views.FinalizeAuthView.as_view(), name="finalize"),
    path("authenticate", views.get_scope_permission, name="authenticate"),
]



from django.test import RequestFactory, TestCase
import jwt
from django.conf import settings
from ..middleware import authenticate
from django.contrib.auth import get_user_model


def generate_jwt_token(shop):
    return jwt.encode(
        {
            "iss": f"https://{shop.myshopify_domain}",
            "dest": f"https://{shop.myshopify_domain}",
        },
        settings.SHOPIFY_APP_API_SECRET,
    )


class ShopifyTokenAuthenticationTest(TestCase):
    def setUp(self):
        settings.AUTH_USER_MODEL = "shopify_auth.AuthAppShopUser"
        self.myshopify_domain = "test-1.myshopify.com"
        self.user = get_user_model().objects.create(
            myshopify_domain=self.myshopify_domain
        )
        self.factory = RequestFactory()
        self.valid_token = f"Bearer {generate_jwt_token(self.user)}"

    def test_valid_token(self):
        request = self.factory.get("/", HTTP_AUTHORIZATION=self.valid_token)
        user = authenticate(request)
        self.assertEqual(user.myshopify_domain, self.myshopify_domain)
        self.assertEqual(self.user.id, user.id)

    def test_not_existing_app_installation_valid_token(self):
        self.user.delete()
        request = self.factory.get("/", HTTP_AUTHORIZATION=self.valid_token)
        user = authenticate(request)
        self.assertIsNone(user)

    def test_invalid_token(self):
        jwt_token = "Bearer not_jwt"
        request = self.factory.get("/", HTTP_AUTHORIZATION=jwt_token)
        user = authenticate(request)
        self.assertIsNone(user)

    def test_missing_header(self):
        request = self.factory.get("/")
        result = authenticate(request)
        self.assertIsNone(result)

    def test_empty_header(self):
        request = self.factory.get("/", HTTP_AUTHORIZATION="")
        result = authenticate(request)
        self.assertIsNone(result)



import logging
from urllib.parse import urlparse

from rest_framework.authentication import BaseAuthentication, get_authorization_header
from rest_framework.exceptions import AuthenticationFailed

from jose import jwt
from jose.exceptions import ExpiredSignatureError, JWTClaimsError, JWTError

from django.conf import settings
from django.contrib.auth import get_user_model

INVALID_TOKEN_MESSAGE = "Invalid JWT Token"


class ShopifyTokenAuthentication(BaseAuthentication):
    keyword = "Bearer"

    @staticmethod
    def get_hostname(url):
        return urlparse(url).netloc

    def authenticate(self, request):
        UserModel = get_user_model()
        auth = get_authorization_header(request).split()
        if not auth or auth[0].lower() != self.keyword.lower().encode():
            return None
        if len(auth) == 1:
            msg = "Invalid token header. No credentials provided."
            raise AuthenticationFailed(msg)
        elif len(auth) > 2:
            msg = "Invalid token header. Token string should not contain spaces."
            raise AuthenticationFailed(msg)

        try:
            token = auth[1].decode()
        except UnicodeError:
            msg = "Invalid token header. Token string should not contain invalid characters."
            raise AuthenticationFailed(msg)

        try:
            decoded_payload = jwt.decode(
                token,
                settings.SHOPIFY_APP_API_SECRET,
                algorithms=["HS256"],
                audience=settings.SHOPIFY_APP_API_KEY,
                options={"verify_sub": False, "verify_nbf": False},
            )
            dest_host = self.get_hostname(decoded_payload["dest"])
            iss_host = self.get_hostname(decoded_payload["iss"])
            if dest_host != iss_host:
                raise AuthenticationFailed(INVALID_TOKEN_MESSAGE)

            try:
                return UserModel.objects.get(myshopify_domain=dest_host)
            except UserModel.DoesNotExist:
                raise AuthenticationFailed(INVALID_TOKEN_MESSAGE)

        except (ExpiredSignatureError, JWTError, JWTClaimsError) as e:
            logging.warning(f"Login user failed: {e}.")
            raise AuthenticationFailed(INVALID_TOKEN_MESSAGE)



import logging
import re
from urllib.parse import urlparse
from django.core.exceptions import ObjectDoesNotExist
from django.contrib.auth import get_user_model

from django.conf import settings
from jose import jwt
from jose.exceptions import ExpiredSignatureError, JWTClaimsError, JWTError


def get_hostname(url):
    return urlparse(url).netloc


def authenticate(request):
    jwt_token = request.headers.get("Authorization")

    if not jwt_token:
        return

    striped_jwt_token = re.sub(r"Bearer\s", "", jwt_token)
    try:
        decoded_payload = jwt.decode(
            striped_jwt_token,
            settings.SHOPIFY_APP_API_SECRET,
            algorithms=["HS256"],
            audience=settings.SHOPIFY_APP_API_KEY,
            options={"verify_sub": False, "verify_nbf": False},
        )

        dest_host = get_hostname(decoded_payload["dest"])
        iss_host = get_hostname(decoded_payload["iss"])

        assert (
            dest_host == iss_host
        ), "'dest' claim host does not match 'iss' claim host"
        assert dest_host, "'dest' claim host not a valid shopify host"

        return get_user_model().objects.get(myshopify_domain=dest_host)

    except (
        ExpiredSignatureError,
        JWTError,
        JWTClaimsError,
        AssertionError,
        ObjectDoesNotExist,
    ) as e:
        logging.warning(f"Login user failed: {e}.")


class SessionTokensAuthMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        user = authenticate(request)
        if user:
            request.user = user
        return self.get_response(request)



from unittest.mock import patch

from django.shortcuts import reverse
from shopify_auth.tests.test_user import ViewsTestCase

from django.contrib.auth import get_user_model
import shopify


def request_token(self, params):
    self.token = "TOKEN"


class FinalizeViewTest(ViewsTestCase):
    def setUp(self):
        super().setUp()
        self.shop_patcher = patch("shopify.Shop", autospec=True)
        mck = self.shop_patcher.start()
        mck.current().currency = "CZK"
        self.addCleanup(self.shop_patcher.stop)
        self.url = reverse("session_tokens:finalize") + "?shop=random_shop"

    @patch.object(shopify.Session, "request_token", request_token)
    def test_creates_user(self):
        response = self.client.get(self.url)
        AuthAppShopUser = get_user_model()
        self.assertTrue(
            AuthAppShopUser.objects.filter(
                myshopify_domain="random_shop.myshopify.com"
            ).exists()
        )
        self.assertEqual(response.status_code, 302)



import logging

from django.conf import settings
from django.http.response import HttpResponse, HttpResponseRedirect
from django.shortcuts import render, reverse
from django.views.generic.base import View

import shopify

from django.contrib.auth import get_user_model


from shopify_auth.views import get_return_address


def get_scope_permission(request, myshopify_domain):
    redirect_uri = request.build_absolute_uri(reverse("session_tokens:finalize"))
    myshopify_domain = myshopify_domain.strip()
    permission_url = shopify.Session(
        myshopify_domain,
        getattr(settings, "SHOPIFY_APP_API_VERSION", "unstable"),
    ).create_permission_url(settings.SHOPIFY_APP_API_SCOPE, redirect_uri)

    return render(
        request,
        "shopify_auth/iframe_redirect.html",
        {"shop": myshopify_domain, "redirect_uri": permission_url},
    )


class FinalizeAuthView(View):
    def get(self, request):
        myshopify_domain = request.GET.get("shop")
        try:
            shopify_session = shopify.Session(
                myshopify_domain,
                getattr(settings, "SHOPIFY_APP_API_VERSION", "unstable"),
            )
            shopify_session.request_token(request.GET)
        except:
            logging.exception("Shopify login failed.")
            return HttpResponse("Shopify login failed.")
        shop, created = get_user_model().objects.update_or_create(
            myshopify_domain=shopify_session.url,
            defaults={"token": shopify_session.token},
        )

        shop.install(request)

        return HttpResponseRedirect(
            get_return_address(request) + f"?shop={myshopify_domain}"
        )



from django.test import Client, TestCase
from django.test.client import RequestFactory

from django.contrib.auth import get_user_model
from shopify_auth.session_tokens.views import get_scope_permission


class GetScopePermissionTest(TestCase):
    def test_get_scope_permission(self):
        self.client = Client()
        rf = RequestFactory()
        self.myshopify_domain = "test-1.myshopify.com"
        shop = get_user_model().objects.create(myshopify_domain=self.myshopify_domain)
        request = rf.get(f"/?shop={shop.myshopify_domain}")
        request.user = shop
        response = get_scope_permission(request, shop.myshopify_domain)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, shop.myshopify_domain)
